#include "slideface.h"

#include "slideworld.h"
#include "slidepoint.h"
#include "slidesurface.h"

BOOL CSLIDEFace::m_bBackFaceCullingEnabled = TRUE;
BOOL CSLIDEFace::m_bClipEnabled = TRUE;
BOOL CSLIDEFace::m_bRenderNormalsEnabled = FALSE;
FLOAT CSLIDEFace::m_fNormalsRenderLength = (FLOAT)0.2;

CListInt *CSLIDEFace::m_plistVertices = NULL;
UINT_32 CSLIDEFace::m_uiCount = 0;

//////////////////////////////////////////////////////////////////////////////
// CSLIDEFace Class
//

CSLIDEFace::CSLIDEFace()
{
  m_uiCount++;
  if ( m_plistVertices == NULL )
    {
      m_plistVertices = new CListInt;
      ASSERT( m_plistVertices != NULL );
      m_plistVertices->Init();
    }

  m_pcSurfaceID = NULL;
  m_pCSLIDESurface = NULL;
  m_plistPointIDs = NULL;
}

CSLIDEFace::~CSLIDEFace()
{
  m_uiCount--;
  if ( m_uiCount == 0 )
    {
      if ( m_plistVertices != NULL )
	{
	  delete m_plistVertices;
	  m_plistVertices = NULL;
	}
    }
}

VOID CSLIDEFace::Init(CHAR *pcFile, UINT_32 uiLine, UINT_32 uiColumn, CHAR *pcID, 
		      CListInt *plistPointIDs)
{
  CSLIDEPrimitive::Init(pcFile, uiLine, uiColumn, pcID);

  ASSERT( plistPointIDs != NULL );

  m_pcSurfaceID = gcppcSLIDEReservedWords[sliderwtSLF_INHERIT];
  m_pCSLIDESurface = NULL;

  m_plistPointIDs = plistPointIDs;
  m_listCSLIDEPoint.Init();
}

VOID CSLIDEFace::Uninit()
{
  CSLIDEPrimitive::Uninit();

  if ( m_plistPointIDs != NULL )
    {
      delete m_plistPointIDs;
      m_plistPointIDs = NULL;
    }
}

BOOL CSLIDEFace::Initialized()
{
  if ( !CSLIDEPrimitive::Initialized() )
    {
      return FALSE;
    }

  return ( m_pcSurfaceID != NULL &&
	   m_plistPointIDs != NULL );
}

BOOL CSLIDEFace::CheckSemantics(CSLIDEWorld *pCSLIDEWorld)
{
  BOOL bCorrect;
  CListIterInt iter;
  CHAR *pcID;
  CSLIDEPoint *pCSLIDEPoint;

  if ( !CSLIDEPrimitive::CheckSemantics(pCSLIDEWorld) )
    {
      return FALSE;
    }

  bCorrect = TRUE;

  // Check Surface Reference

  m_pCSLIDESurface = pCSLIDEWorld->PeekSurface(m_pcSurfaceID);
  if ( m_pCSLIDESurface == NULL )
    {
      HandleError("Undefined %s \"%s\"", 
		  gcppcSLIDEReservedWords[sliderwtSurface],
		  m_pcSurfaceID);
      bCorrect = FALSE;
    }

  // Check Point References

  iter.Init(m_plistPointIDs);

  for ( pcID = (CHAR*)(iter.PeekFirst());
	pcID != NULL;
	pcID = (CHAR*)(iter.PeekNext()) )
    {
      pCSLIDEPoint = pCSLIDEWorld->PeekPoint(pcID);
      if ( pCSLIDEPoint == NULL )
	{
	  HandleError("Undefined %s \"%s\"", gcppcSLIDEReservedWords[sliderwtPoint], 
		      pcID);
	  bCorrect = FALSE;
	}
      m_listCSLIDEPoint.InsertLast(pCSLIDEPoint, 0);
    }

  return bCorrect;
}

BOOL CSLIDEFace::Valid()
{
  if ( !CSLIDEPrimitive::Valid() )
    {
      return FALSE;
    }

  return (GetPointCount() >= 3);
}

CSLIDEPrimitiveIter *CSLIDEFace::AllocateIter()
{
  CSLIDEFaceIter *pCSLIDEFaceIter;

  pCSLIDEFaceIter = new CSLIDEFaceIter;
  ASSERT(pCSLIDEFaceIter != NULL );
  pCSLIDEFaceIter->Init(this);

  return pCSLIDEFaceIter;
}

VOID CSLIDEFace::FreeIter(CSLIDEPrimitiveIter *pCSLIDEFaceIter)
{
  delete pCSLIDEFaceIter;
}

VOID CSLIDEFace::InsertIntoBound(CBound *pCBound)
{
  CListIterInt iter;
  CSLIDEPoint *pCSLIDEPoint;

  ASSERT( pCBound != NULL );

  iter.Init(&m_listCSLIDEPoint);

  for ( pCSLIDEPoint = (CSLIDEPoint*)iter.PeekFirst(); 
	pCSLIDEPoint != NULL;
	pCSLIDEPoint = (CSLIDEPoint*)iter.PeekNext() )
    {
      pCSLIDEPoint->InsertIntoBound(pCBound);
    }
}

//////////////////////////////////////////////////////////////////////////////
// START: ASSIGNMENT 5: Fix this!!!
// 
VOID CSLIDEFace::Preprocess()
{
  CheckDynamic();
  UpdateState();

  // All faces must calculate their normal plane equation at least once
  // Compute m_vNormal and m_ptCentroid
  // Make sure to compute the d component of the plane equation in 
  // m_vNormal

  // m_vNormal = ???
  m_vNormal.Null();
  // m_ptCentroid = ???
  m_ptCentroid.Set(0, 0, 0);
}
// 
// END  : ASSIGNMENT 5: Fix this!!!
//////////////////////////////////////////////////////////////////////////////

VOID CSLIDEFace::CheckDynamic()
{
  CListIterInt iter;
  CSLIDEPoint *pCSLIDEPoint;
  BOOL bBounded;
  BOOL bBoundDynamic;

  bBounded = GetDynamic();
  bBoundDynamic = GetDynamic();

  iter.Init(&m_listCSLIDEPoint);

  for ( pCSLIDEPoint = (CSLIDEPoint*)iter.PeekFirst(); 
	pCSLIDEPoint != NULL;
	pCSLIDEPoint = (CSLIDEPoint*)iter.PeekNext() )
    {
      bBounded = TRUE;
      if ( pCSLIDEPoint->GetBoundDynamic() )
	{
	  bBoundDynamic = TRUE;
	  break;
	}
    }

  SetBounded(bBounded);
  SetBoundDynamic(bBoundDynamic);
  SetDynamic(bBoundDynamic);
  ASSERT( !(!GetBounded() && GetBoundDynamic()) );
}

//////////////////////////////////////////////////////////////////////////////
// START: ASSIGNMENT 5: Fix this!!!
// 
VOID CSLIDEFace::UpdateState()
{
  if ( GetBoundDynamic() )
    {
      // All faces must calculate their normal plane equation at least once
      // Compute m_vNormal and m_ptCentroid
      // Make sure to compute the d component of the plane equation in 
      // m_vNormal
      
      // m_vNormal = ???
      m_vNormal.Null();
      // m_ptCentroid = ???
      m_ptCentroid.Set(0, 0, 0);
  
    }
}

// This should assign m_vNormal just the direction = [a, b, c, 0].  It does 
// not completely compute the face's plane equation.
// This uses Newell's Algorithm for finding the best fit planar equation
// for a set of points in 3 space (pg. 476 Foley and van Dam)
// I modified it so that the normal would be positive for counter-clockwise
// polygons, and I removed the 1/2 coefficient because we are concerned 
// with relative areas, not absolute areas.

VOID CSLIDEFace::CalculateNormal()
{
  m_vNormal.Null();
}

// Assign m_ptCentroid the average of the points in the polygon contour
VOID CSLIDEFace::CalculateCentroid()
{
  m_ptCentroid.Null();
}

VOID CSLIDEFace::Render(const Matrix &mGvp_proj, 
			const Matrix &mGproj_vrc, 
			const Matrix &mGvrc_proj, 
			const Matrix &mGvrc_obj, 
			const Matrix &mGobj_vrc, 
			CBound *pCBoundClip, 
			CListInt * /*plistCSLIDELightPath*/,
			CSLIDESurface *pCSLIDESurface,
			SLIDEShadingType /*eShading*/, SLIDELODType eLOD, 
			SLIDESolidType eSolid)
{
  GLenum mode;
  CListIterInt iter;
  CSLIDEPoint *pCSLIDEPoint;
  Matrix mGproj_obj;
  Vertex *pvx;
      
  mGproj_obj = mGproj_vrc * mGvrc_obj;

  // Check Back Face Culling, when do we do that again?
  if ( GetBackFaceCullingEnabled() )
    {
    }

  // Modify Rendering Properties
  ASSERT( eLOD == slidelodtSLF_EDGES || eLOD == slidelodtSLF_FULL );
  ModifySurfacePointer(pCSLIDESurface);

  // Build a list of temporary Vertex structures which we dynamically allocate.
  // This will be the input to the clipping routine in the 
  // PROJ canonical clipping coordinate system
  iter.Init(&m_listCSLIDEPoint);

  for ( pCSLIDEPoint = (CSLIDEPoint*)iter.PeekFirst(); 
	pCSLIDEPoint != NULL;
	pCSLIDEPoint = (CSLIDEPoint*)iter.PeekNext() )
    {
      pvx = CBound::AllocateVertex();
      ASSERT( pvx != NULL );

      pvx->SetPoint() = mGproj_obj * pCSLIDEPoint->GetPoint();

      m_plistVertices->InsertLast(pvx, 0);
    }

  // Clip the polygon against the bound if clipping is enabled
  // and we do not lie entirely within the camera's view frustum
  if ( GetClipEnabled() && pCBoundClip != NULL )
    {
      // The clipping routine should modify the list of vertices.
      // It should deallocate any vertices it removes by calling 
      // CBound::DeallocateVertex(pvx) on them.  It should also
      // insert any new vertices into the m_plistVertices, so that
      // when the ClipPolygon routine finishes the list is the clipped
      // polygon contour.
      pCBoundClip->ClipPolygon(*m_plistVertices);
    }

  // Transform the clipped vertices into pixel coords and render.
  // We remove the vertices from the list of vertices and deallocate 
  // these temporary vertex structures as we go.
  gilColor3fv(pCSLIDESurface->GetColor().Get());
  gilBegin(GL_LINE_LOOP);
  {
    for ( pvx = (Vertex*)m_plistVertices->RemoveFirst(); 
	  pvx != NULL;
	  pvx = (Vertex*)m_plistVertices->RemoveFirst() )
      {
	pvx->SetPoint() = mGvp_proj * pvx->GetPoint();
	
	pvx->Render();
	CBound::DeallocateVertex(pvx);
      }
  }
  gilEnd();

  // Render the normal if desired
  if ( GetRenderNormalsEnabled() )
    {
      gilPushAttrib(GL_CURRENT_BIT);
      {
	gilColor3f(1.0, 0.0, 1.0);
	gilBegin(GL_LINES);
	{
	  RenderNormal(mGvp_proj, mGproj_vrc, mGvrc_obj, mGobj_vrc, 
		       m_ptCentroid, m_vNormal);
	}
	gilEnd();
      }
      gilPopAttrib();
    }
}
// 
// END  : ASSIGNMENT 5: Fix this!!!
//////////////////////////////////////////////////////////////////////////////

VOID CSLIDEFace::ModifySurfacePointer(CSLIDESurface* &pCSLIDESurface)
{
  if ( m_pCSLIDESurface != CSLIDESurface::GetSLF_INHERIT() )
    {
      pCSLIDESurface = m_pCSLIDESurface;
    }
}

//////////////////////////////////////////////////////////////////////////////
// START: ASSIGNMENT 5: Fix this!!!
// 
VOID CSLIDEFace::RenderNormal(const Matrix &mGvp_proj, const Matrix &mGproj_vrc, 
			      const Matrix &mGvrc_obj, const Matrix &mGobj_vrc,
			      const Point &pt, const Vector &vNormal)
{
  Point ptStart, ptEnd;
  Vector v;
  Matrix m;
  FLOAT fDeterminant;

  // We need v = normal in VRC, how do we transform normals?
  v = vNormal; // ???

  ptStart = mGvrc_obj * pt;

  v[W] = 0;
  v.Normalize();
  ptEnd = ptStart + GetNormalsRenderLength() * v;

  // We break here because we want to see the shortening effects of the 
  // projection on the normal

  ptStart = mGvp_proj * mGproj_vrc * ptStart;
  ptEnd = mGvp_proj * mGproj_vrc * ptEnd;

  gilVertex3fv(ptStart.Get());
  gilVertex3fv(ptEnd.Get());
}
// 
// END  : ASSIGNMENT 5: Fix this!!!
//////////////////////////////////////////////////////////////////////////////

VOID CSLIDEFace::WriteSLIDE(FILE *fpOutputFile)
{
  CListIterInt iter;
  CSLIDEPoint *pCSLIDEPoint;

  ASSERT( fpOutputFile != NULL );

  fprintf(fpOutputFile, "%s %s ( ", gcppcSLIDEReservedWords[sliderwtFace], 
	  m_pcID);

  iter.Init(&m_listCSLIDEPoint);

  for ( pCSLIDEPoint = (CSLIDEPoint*)iter.PeekFirst(); 
	pCSLIDEPoint != NULL;
	pCSLIDEPoint = (CSLIDEPoint*)iter.PeekNext() )
    {
      fprintf(fpOutputFile, "%s ", pCSLIDEPoint->GetID());      
    }

  fprintf(fpOutputFile, ")\n");

  if ( m_pcSurfaceID != gcppcSLIDEReservedWords[sliderwtSLF_INHERIT] )
    {
      fprintf(fpOutputFile, "  %s %s\n", 
	      gcppcSLIDEReservedWords[sliderwtSurface], 
	      m_pcSurfaceID);
    } 

  fprintf(fpOutputFile, "%s\n", 
	  gcppcSLIDEReservedWords[sliderwtEndFace]);
}

UINT_32 CSLIDEFace::GetPointCount()
{
  return m_listCSLIDEPoint.GetLength();
}

//////////////////////////////////////////////////////////////////////////////
// CSLIDEFaceIter Class
//

CSLIDEFaceIter::CSLIDEFaceIter()
{
}

CSLIDEFaceIter::~CSLIDEFaceIter()
{
}

VOID CSLIDEFaceIter::Init(CSLIDEFace *pCSLIDEFace)
{
  CSLIDEPrimitiveIter::Init(pCSLIDEFace);

  m_iter.Init(&(pCSLIDEFace->m_listCSLIDEPoint));
}

VOID CSLIDEFaceIter::Uninit()
{
  CSLIDEPrimitiveIter::Uninit();
}

CSLIDEPoint *CSLIDEFaceIter::PeekFirstCSLIDEPoint()
{
  return (CSLIDEPoint*)(m_iter.PeekFirst());
}

CSLIDEPoint *CSLIDEFaceIter::PeekNextCSLIDEPoint()
{
  return (CSLIDEPoint*)(m_iter.PeekNext());
}

CSLIDEPoint *CSLIDEFaceIter::PeekLastCSLIDEPoint()
{
  return (CSLIDEPoint*)(m_iter.PeekLast());
}

CSLIDEPoint *CSLIDEFaceIter::PeekPrevCSLIDEPoint()
{
  return (CSLIDEPoint*)(m_iter.PeekPrev());
}
