#ifndef __SLIDEGROUP_H__
#define __SLIDEGROUP_H__

#include "slidenode.h"

class CSLIDEWorld;
class CSLIDESurface;
class CSLIDEInstance;

//////////////////////////////////////////////////////////////////////////////
// CSLIDEGroup Class
//

class CSLIDEGroupIter;

class CSLIDEGroup : public CSLIDENode
{
//////////////////////////////////////////////////
// Friends
//
public:
  friend class CSLIDEGroupIter;

//////////////////////////////////////////////////
// Enumerated Types
//

//////////////////////////////////////////////////
// Initialization
//
public:
  CSLIDEGroup();
  virtual ~CSLIDEGroup();
  virtual VOID Init(CHAR *pcFile, UINT_32 uiLine, UINT_32 uiColumn, 
		    CHAR *pcID);
  virtual VOID Uninit();

  virtual BOOL Initialized();

  virtual VOID InsertInstance(CSLIDEInstance *pCSLIDEInstance);
  virtual BOOL CheckSemantics(CSLIDEWorld *pCSLIDEWorld);

  virtual BOOL Valid();

//////////////////////////////////////////////////
// Iteration Functions
//
public:
  virtual CSLIDEPrimitiveIter *AllocateIter();
  virtual VOID FreeIter(CSLIDEPrimitiveIter *pCSLIDEGroupIter);

//////////////////////////////////////////////////
// Geometric Computations
//
public:
  virtual CSLIDECamera *FindCamera(CListIterInt *piterIDs, 
				   CListInt *plistCSLIDEInstance);
  virtual CSLIDELight *FindLight(CListIterInt *piterIDs, 
				 CListInt *plistCSLIDEInstance);

  virtual VOID Preprocess();
  virtual VOID PreprocessGraph();

  virtual VOID Update();
  virtual VOID UpdateGraph();
  virtual VOID InsertIntoBound(const Matrix &m, CBound &bound);

protected:
  virtual VOID CheckDynamic();
  virtual VOID UpdateState();
  virtual VOID CalculateBound();

//////////////////////////////////////////////////
// Rendering
//
public:

protected:
  virtual VOID ModifySurfacePointer(CSLIDESurface* &pCSLIDESurface);
  virtual VOID ModifyShadingFlag(SLIDEShadingType &eShading);
  virtual VOID ModifyLODFlag(SLIDELODType &eLOD);
  virtual CBound::BoundIntersectionType CheckBoundReject(const Matrix &m, 
							 CBound *pCBound);
  virtual VOID RenderGeometry(const Matrix &mGvp_proj, 
			      const Matrix &mGproj_vrc, const Matrix &mGvrc_proj, 
			      const Matrix &mGvrc_obj, const Matrix &mGobj_vrc, 
			      CBound *pCBoundClip, 
			      CListInt *plistLights,
			      CSLIDESurface *pCSLIDESurface,
			      SLIDEShadingType eShading, 
			      SLIDELODType eLOD);

public:
  virtual VOID RenderBound(const Matrix &mGvp_proj, const Matrix &mGproj_vrc, 
			   const Matrix &mGvrc_obj);

//////////////////////////////////////////////////
// Output Functions
//
public:
  virtual VOID WriteSLIDE(FILE *fpOutputFile);

//////////////////////////////////////////////////
// Accessor Functions
//
public:
  virtual inline VOID SetSurfaceID(CHAR *pcSurfaceID);
  virtual inline VOID SetShading(DynInt *pdyniShading);
  virtual inline VOID SetLOD(DynInt *pdyniLOD);

  virtual inline NodeType GetNodeType();

  virtual inline const CBound *GetCBound();
  virtual inline BOOL GetBoundEmpty();

  virtual UINT_32 GetInstanceCount();

//////////////////////////////////////////////////
// Class Variables
//
protected:

//////////////////////////////////////////////////
// Member Variables
//
protected:
  CHAR *m_pcSurfaceID;
  CSLIDESurface *m_pCSLIDESurface;
  DynInt m_dyniShading;
  DynInt m_dyniLOD;
  
  CListInt m_listCSLIDEInstance;

  CBound m_bound;
};

//////////////////////////////////////////////////////////////////////
// Inlined Functions
//

inline VOID CSLIDEGroup::SetSurfaceID(CHAR *pcSurfaceID)
{
  m_pcSurfaceID = pcSurfaceID;
}

inline VOID CSLIDEGroup::SetShading(DynInt *pdyniShading)
{
  m_dyniShading.Set(pdyniShading);
}

inline VOID CSLIDEGroup::SetLOD(DynInt *pdyniLOD)
{
  m_dyniLOD.Set(pdyniLOD);
}

inline CSLIDENode::NodeType CSLIDEGroup::GetNodeType()
{
  return ntGroup;
}

inline const CBound *CSLIDEGroup::GetCBound()
{
  return &m_bound;
}

inline BOOL CSLIDEGroup::GetBoundEmpty()
{
  return m_bound.GetEmpty();
}

//////////////////////////////////////////////////////////////////////
// CSLIDEGroupIter Class
//

class CSLIDEGroupIter : public CSLIDEPrimitiveIter
{
//////////////////////////////////////////////////
// Initialization
//
public:
  CSLIDEGroupIter();
  virtual ~CSLIDEGroupIter();
  virtual VOID Init(CSLIDEGroup *pCSLIDEGroup);
  virtual VOID Uninit();

//////////////////////////////////////////////////
// Iteration Functions
//
public:
  virtual CSLIDEInstance *PeekFirstCSLIDEInstance();
  virtual CSLIDEInstance *PeekNextCSLIDEInstance();

//////////////////////////////////////////////////
// Member Variables
//
protected:
  CListIterInt m_iter;
};

//////////////////////////////////////////////////////////////////////
// Inlined Functions
//


#endif // __SLIDEGROUP_H__
