#ifndef __SLIDERENDER_H__
#define __SLIDERENDER_H__

#include "slideprimitive.h"

class CSLIDEWorld;
class CSLIDECameraPath;
class CSLIDENode;

class CSLIDETclVariables;

//////////////////////////////////////////////////////////////////////
// CSLIDERender Class
//

class CSLIDERender : public CSLIDEPrimitive
{
//////////////////////////////////////////////////
// Friends
//
public:

//////////////////////////////////////////////////
// Enumerated Types
//

//////////////////////////////////////////////////
// Initialization
//
public:
  CSLIDERender();
  virtual ~CSLIDERender();
  virtual VOID Init(CHAR *pcFile, UINT_32 uiLine, UINT_32 uiColumn, 
		    CHAR *pcViewportID, CSLIDECameraPath *pCSLIDECameraPath,
		    CHAR *pcWorldRootID);
  virtual VOID Uninit();

  virtual BOOL Initialized();

  virtual inline VOID SetTclTag(CHAR *pcTclTag);
  virtual inline VOID SetLOD(DynInt *pdyniLOD);

  virtual BOOL CheckSemantics(CSLIDEWorld *pCSLIDEWorld);

  virtual BOOL Valid();

public:  

//////////////////////////////////////////////////
// Iteration Functions
//
public:

//////////////////////////////////////////////////
// Geometric Computations
//
public:
  virtual VOID PreprocessGraph();
  virtual VOID UpdateGraph();

protected:
  virtual VOID CheckDynamic();
  virtual VOID UpdateState();

//////////////////////////////////////////////////
// Input Functions
//
public:
  VOID HandleReshape(const Vector &vSize);
//////////////////////////////////////////////////////////////////////////////
// START: ASSIGNMENT 5: Crystal Ball UI
// 
  BOOL HandleMouseButton(const Vector &vSize,
			 INT_32 iButton, INT_32 iState, 
			 const Vector &vClick);
  BOOL HandleMouseMotion(const Vector &vSize,
			 const Vector &vClick);
// 
// END  : ASSIGNMENT 5: Crystal Ball UI
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////
// Rendering
//
public:
//////////////////////////////////////////////////////////////////////////////
// START: ASSIGNMENT 5: Deal with the camera
// 
  virtual VOID Render(const Vector &vOrigin, const Vector &vSize);
// 
// END  : ASSIGNMENT 5: Deal with the camera
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////
// Output Functions
//
public:
  virtual VOID WriteSLIDE(FILE *fpOutputFile);

//////////////////////////////////////////////////
// Accessor Functions
//
public:
  static inline BOOL GetRenderClipBound();
  static inline VOID SetRenderClipBound(BOOL bRenderClipBound);
  static inline BOOL GetCrystalBallEnabled();
  static inline VOID SetCrystalBallEnabled(BOOL bCrystalBallEnabled);

//////////////////////////////////////////////////
// Class Variables
//
protected:
  static BOOL m_bRenderClipBound;
  static BOOL m_bCrystalBallEnabled;

  static UINT_32 m_uiCount;
  static Vector m_vProjMin;
  static Vector m_vProjMax;
  static CBound m_boundClip;

  static CListInt *m_plistCSLIDETclVariables;

//////////////////////////////////////////////////
// Member Variables
//
protected:
  CHAR *m_pcTclTag;
  CSLIDETclVariables *m_pCSLIDETclVariables;

  CHAR *m_pcViewportID;

  CSLIDECameraPath *m_pCSLIDECameraPath; // Traversal to Camera and the Camera

  CHAR *m_pcWorldRootID;
  CSLIDENode *m_pWorldRoot;

  DynInt m_dyniLOD;

//////////////////////////////////////////////////////////////////////////////
// START: ASSIGNMENT 5: Add what ever variables you need
// 
  // Crystal Ball Variables
  Matrix m_mBall;        // The current crystal ball matrix (scales and rotates)
  Matrix m_mBallInverse; // The inverse of the current crystal ball matrix

  INT_32 m_iButtonCurr;  // The mouse button which is currently depressed
  Vector m_vStart;       // The starting click position (useful for Right button)
  Vector m_vLatest;      // The previous click position (useful for Left button)
  FLOAT m_fScaleLast;    // Last scale value for middle mouse button
// 
// END  : ASSIGNMENT 5: Add what ever variables you need
//////////////////////////////////////////////////////////////////////////////
};

//////////////////////////////////////////////////////////////////////
// Inlined Functions
//

inline VOID CSLIDERender::SetTclTag(CHAR *pcTclTag)
{
  m_pcTclTag = pcTclTag;
}

inline VOID CSLIDERender::SetLOD(DynInt *pdyniLOD)
{
  m_dyniLOD.Set(pdyniLOD);
}

inline BOOL CSLIDERender::GetRenderClipBound()
{
  return m_bRenderClipBound;
}

inline VOID CSLIDERender::SetRenderClipBound(BOOL bRenderClipBound)
{
  m_bRenderClipBound = bRenderClipBound;
}

inline BOOL CSLIDERender::GetCrystalBallEnabled()
{
  return m_bCrystalBallEnabled;
}

inline VOID CSLIDERender::SetCrystalBallEnabled(BOOL bCrystalBallEnabled)
{
  m_bCrystalBallEnabled = bCrystalBallEnabled;
}

//////////////////////////////////////////////////////////////////////
// CSLIDETclVariables Class
//

class CSLIDETclVariables
{
//////////////////////////////////////////////////
// Initialization
//
public:
  CSLIDETclVariables();
  virtual ~CSLIDETclVariables();
  virtual VOID Init(CHAR *pcID);
  virtual VOID Uninit();

  virtual BOOL Initialized();
  virtual BOOL Valid();

//////////////////////////////////////////////////
// Accessor Functions
//
public:
  inline CHAR *GetID() const;

  inline FLOAT GetWidth() const;
  VOID SetWidth(FLOAT fWidth);
  inline FLOAT GetHeight() const;
  VOID SetHeight(FLOAT fHeight);
  inline FLOAT GetMouseX() const;
  VOID SetMouseX(FLOAT fMouseX);
  inline FLOAT GetMouseY() const;
  VOID SetMouseY(FLOAT fMouseY);

//////////////////////////////////////////////////
// Member Variables
//
protected:
  CHAR *m_pcID;

  // Window Variables
  FLOAT m_fWidth;
  FLOAT m_fHeight;
  FLOAT m_fMouseX;
  FLOAT m_fMouseY;
};

//////////////////////////////////////////////////////////////////////
// Inlined Functions
//

inline CHAR *CSLIDETclVariables::GetID() const
{
  return m_pcID;
}

inline FLOAT CSLIDETclVariables::GetWidth() const
{
  return m_fWidth;
}

inline FLOAT CSLIDETclVariables::GetHeight() const
{
  return m_fHeight;
}

inline FLOAT CSLIDETclVariables::GetMouseX() const
{
  return m_fMouseX;
}

inline FLOAT CSLIDETclVariables::GetMouseY() const
{
  return m_fMouseY;
}

#endif // __SLIDERENDER_H__
